--imports
local assert = require("justo.assert")
local justo = require("justo")
local suite, test, init = justo.suite, justo.test, justo.init
local Parser = require("dogma.syn.Parser")
local SentType = require("dogma.syn.SentType")
local DirectiveType = require("dogma.syn.DirectiveType")

--Suite.
return suite("dogma.syn.DirectiveParser", function()
  local parser, dir

  ----------
  -- init --
  ----------
  init("*", function()
    parser = Parser.new()
  end):title("Create parser")

  -------------
  -- #!env --
  -------------
  test("#!env", function()
    parser:parse("#!env\nx = 1")
    dir = parser:next()
    assert(dir):isTable():has({
      line = 1,
      col = 1,
      type = SentType.DIRECTIVE,
      subtype = DirectiveType.ENV
    })
  end)

  -------------
  -- #!react --
  -------------
  test("#!react", function()
    parser:parse("#!react\nx = 1")
    dir = parser:next()
    assert(parser._.options.react):eq(true)
    assert(dir):isTable():has({
      line = 1,
      col = 1,
      type = SentType.DIRECTIVE,
      subtype = DirectiveType.REACT
    })
  end)

  --------------------
  -- nextCoverage() --
  --------------------
  suite("nextCoverage()", function()
    test("coverage", function()
      parser:parse("#!coverage ignore next")
      dir = parser:next()
      assert(dir):isTable():has({
        line = 1,
        col = 1,
        type = SentType.DIRECTIVE,
        subtype = DirectiveType.COVERAGE,
        ignore = "next"
      })
    end)

    test("cov", function()
      parser:parse("#!cov ignore next")
      dir = parser:next()
      assert(dir):isTable():has({
        line = 1,
        col = 1,
        type = SentType.DIRECTIVE,
        subtype = DirectiveType.COVERAGE,
        ignore = "next"
      })
    end)
  end)

  ------------------
  -- nextInsert() --
  ------------------
  test("nextInsert()", function()
    parser:parse("#!insert /* istanbul ignore next */")
    dir = parser:next()
    assert(dir):isTable():has({
      line = 1,
      col = 1,
      type = SentType.DIRECTIVE,
      subtype = DirectiveType.INSERT,
      code = "/* istanbul ignore next */"
    })
  end)

  --------------
  -- nextIf() --
  --------------
  suite("nextIf()", function()
    test("#!if name then - without else", function()
      parser:parse([[
#!if py then
print("one!")
print("two!")
#!end
]])
      dir = parser:next()
      assert(dir):isTable():has({
        line = 1,
        col = 1,
        type = SentType.DIRECTIVE,
        subtype = DirectiveType.IF,
        cond = {"py"},
        elif = nil,
        el = nil
      })
      assert(dir.body):len(2)
      assert(tostring(dir.body[1])):eq("(call print one!)")
      assert(tostring(dir.body[2])):eq("(call print two!)")
    end)

    test("#!if name1 or name2 then - without else", function()
      parser:parse([[
#!if redis or postgres then
print("one!")
print("two!")
#!end
]])
      dir = parser:next()
      assert(dir):isTable():has({
        line = 1,
        col = 1,
        type = SentType.DIRECTIVE,
        subtype = DirectiveType.IF,
        cond = {"redis", "postgres"},
        elif = nil,
        el = nil
      })
      assert(dir.body):len(2)
      assert(tostring(dir.body[1])):eq("(call print one!)")
      assert(tostring(dir.body[2])):eq("(call print two!)")
    end)

    test("#!if not name then - without else", function()
      parser:parse([[
#!if not py then
print("one!")
print("two!")
#!end
]])
      dir = parser:next()
      assert(dir):isTable():has({
        line = 1,
        col = 1,
        type = SentType.DIRECTIVE,
        subtype = DirectiveType.IF,
        cond = "not py",
        elif = nil,
        el = nil
      })
      assert(dir.body):len(2)
      assert(tostring(dir.body[1])):eq("(call print one!)")
      assert(tostring(dir.body[2])):eq("(call print two!)")
    end)

    test("#!if name then - without end", function()
      parser:parse([[
#!if py then
print("one!")
print("two!")
]])
      assert(function() parser:next() end):raises("'end' expected at the end of code.")
    end)

    test("#!if name then - with else", function()
      parser:parse([[
#!if py then
print("one!")
print("two!")
#!else
print("three!")
print("four!")
#!end
]])
      dir = parser:next()
      assert(dir):isTable():has({
        line = 1,
        col = 1,
        type = SentType.DIRECTIVE,
        subtype = DirectiveType.IF,
        cond = {"py"},
        elif = nil
      })
      assert(dir.body):len(2)
      assert(tostring(dir.body[1])):eq("(call print one!)")
      assert(tostring(dir.body[2])):eq("(call print two!)")
      assert(dir.el):len(2)
      assert(tostring(dir.el[1])):eq("(call print three!)")
      assert(tostring(dir.el[2])):eq("(call print four!)")
    end)

    test("#!if name then - with else if", function()
      parser:parse([[
#!if py then
print("one!")
print("two!")
#!else if js then
print("three!")
print("four!")
#!else
print("five!")
print("six!")
#!end
]])
      dir = parser:next()
      assert(dir):isTable():has({
        line = 1,
        col = 1,
        type = SentType.DIRECTIVE,
        subtype = DirectiveType.IF,
        cond = {"py"}
      })
      assert(dir.body):len(2)
      assert(tostring(dir.body[1])):eq("(call print one!)")
      assert(tostring(dir.body[2])):eq("(call print two!)")
      assert(dir.elif):isNotNil()
      assert(dir.elif.cond):eq("js")
      assert(tostring(dir.elif.body[1])):eq("(call print three!)")
      assert(tostring(dir.elif.body[2])):eq("(call print four!)")
      assert(dir.el):len(2)
      assert(tostring(dir.el[1])):eq("(call print five!)")
      assert(tostring(dir.el[2])):eq("(call print six!)")
    end)

    test("#!if name then - error - nested directive", function()
      parser:parse([[
#!if py then
print("one!")
print("two!")
#!else if js then
print("three!")
print("four!")
#!coverage ignore next
#!end
]])
      assert(function() parser:next() end):raises("anonymous code (7, 1): 'else if' directive can't be nested.")
    end)

    test("#!if name then - error - nested directive", function()
      parser:parse([[
#!if py then
print("one!")
print("two!")
#!if py then
print("three!")
print("four!")
#!end
#!end
]])
      assert(function() parser:next() end):raises("anonymous code (4, 1): 'if' directive can't be nested.")
    end)

    test("#!if name then - error - nested else", function()
      parser:parse([[
#!if py then
print("one!")
print("two!")
#!else
#!if py then
print("three!")
print("four!")
#!end
#!end
]])
      assert(function() parser:next() end):raises("anonymous code (5, 1): 'else' directive can't be nested.")
    end):tags("123")
  end)

  -------------------
  -- nextRunWith() --
  -------------------
  suite("nextRunWith()", function()
    test("#!/usr/bin/env node", function()
      parser:parse("#!/usr/bin/env node")
      dir = parser:next()
      assert(dir):isTable():has({
        line = 1,
        col = 1,
        type = SentType.DIRECTIVE,
        subtype = DirectiveType.RUNWITH,
        cmd = "/usr/bin/env node"
      })
    end)
  end)
end):tags("directive")
